package com.wtkj.service.impl;

import com.wtkj.entity.Project;
import com.wtkj.entity.ProjectAuth;
import com.wtkj.entity.ProjectStage;
import com.wtkj.entity.TemplateTypeAndStage;
import com.wtkj.service.IAsyncService;
import com.wtkj.service.IProjectAuthService;
import com.wtkj.service.IProjectStageService;
import com.wtkj.service.ITypeAndStageService;
import lombok.AllArgsConstructor;
import org.springblade.core.secure.BladeUser;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.DateUtil;
import org.springblade.core.tool.utils.Func;
import org.springblade.system.entity.Dept;
import org.springblade.system.feign.ISysClient;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;

import static com.wtkj.config.MagicValue.ZERO;

/**
 * @author Blizzard
 * @create at 2023-09-13 16:32
 * @describe
 */
@Service
@AllArgsConstructor
public class AsyncServiceImpl implements IAsyncService {

	private final ITypeAndStageService typeAndStageService;
	private final IProjectStageService stageService;
	private final IProjectAuthService projectAuthService;
	private final ISysClient sysClient;


	@Override
	@Async("asyncPoolTaskExecutor")
	public void createProject(Project project, BladeUser user) {
		//1.项目类型  --> 使用类型对应的模版阶段创建项目阶段
		Long projectType = project.getProjectType();
		List<TemplateTypeAndStage> templateStage = typeAndStageService.getStages(projectType);
		//2.创建项目阶段
		if (!CollectionUtils.isEmpty(templateStage)) {
			List<ProjectStage> stagesList = new ArrayList<>();
			templateStage.forEach(template -> {
				//项目的阶段记录表
				ProjectStage stage = new ProjectStage();
				stage.setProjectId(project.getId());
				stage.setName(template.getName());
				stage.setSort(template.getSort());
				stage.setCreateUser(project.getCreateUser());
				stage.setCreateDept(project.getCreateDept());
				stage.setCreateTime(DateUtil.now());
				stage.setUpdateTime(DateUtil.now());
				stage.setUpdateUser(project.getUpdateUser());
				stagesList.add(stage);
			});
			if (!CollectionUtils.isEmpty(stagesList)) {
				stageService.saveBatch(stagesList);
			}
		}

		//3.如果主管单位和当前登录用户不是同一机构
		String deptIdStr = user.getDeptId();
		if (deptIdStr != null) {
			Long deptId = Func.firstLong(deptIdStr);
			if (!deptId.equals(project.getCompetentUnit())) {
				ProjectAuth auth = new ProjectAuth();
				auth.setUserId(user.getUserId());
				R<Dept> rpc = sysClient.getDept(deptId);
				if (rpc.isSuccess()) {
					String ancestors = rpc.getData().getAncestors();
					if (ancestors != null && ancestors.contains(String.valueOf(ZERO))) {
						List<Long> longList = Func.toLongList(ancestors);
						if (!CollectionUtils.isEmpty(longList) && longList.size() > 2) {
							auth.setTopDept(longList.get(1));
						}
					}
				}
				auth.setUserDept(deptId);
				auth.setProjectId(project.getId());
				projectAuthService.save(auth);
			}
		}

	}

	@Async("asyncPoolTaskExecutor")
	@Override
	public void removeProject(List<Long> longList) {

	}

}
